@@ -16,7 +16,7 @@
   IBOutlet NSTextField *codecField;
   IBOutlet NSDrawer *drawerWindow;
   IBOutlet NSTextField *filesizeField;
-  IBOutlet NSComboBox *genreCombo;
+  IBOutlet NSTextField *genreField;
   IBOutlet NSTextField *lengthField;
 	IBOutlet NSTextField *titleField;
 	IBOutlet NSButton *toggleButton;
