@@ -29,6 +29,7 @@
  *   use big-endian dates.)
  *
  */
+#include "config.h"
 #include "libmtp.h"
 #include "libusb-glue.h"
 #include "device-flags.h"
@@ -204,7 +205,7 @@ static void free_mtpdevice_list(mtpdevice_list_t *devlist)
  *        to this file in human-readable hex so we can scruitinze them.
  * @return 1 if the device is MTP compliant, 0 if not.
  */
-int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
+static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
 {
   usb_dev_handle *devh;
   unsigned char buf[1024], cmd;
@@ -224,35 +225,65 @@ int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
   }
 
   /*
-   * Loop over the device configurations and interfaces. Nokia MTP-capable 
-   * handsets (possibly others) typically have the string "MTP" in their 
-   * MTP interface descriptions, that's how they can be detected, before
-   * we try the more esoteric "OS descriptors" (below).
+   * This sometimes crashes on the j for loop below
+   * I think it is because config is NULL yet 
+   * dev->descriptor.bNumConfigurations > 0
+   * this check should stop this
    */
-  for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
-    uint8_t j;
+  if (dev->config) {
+    /*
+     * Loop over the device configurations and interfaces. Nokia MTP-capable 
+     * handsets (possibly others) typically have the string "MTP" in their 
+     * MTP interface descriptions, that's how they can be detected, before
+     * we try the more esoteric "OS descriptors" (below).
+     */
+    for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+      uint8_t j;
+      
+      for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
+        int k;
+        for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
+          buf[0] = '\0';
+          ret = usb_get_string_simple(devh, 
+              dev->config[i].interface[j].altsetting[k].iInterface, 
+              (char *) buf, 
+              1024);
+    if (ret < 3)
+      continue;
+          if (strcmp((char *) buf, "MTP") == 0) {
+      if (dumpfile != NULL) {
+              fprintf(dumpfile, "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
+        fprintf(dumpfile, "   Interface description contains the string \"MTP\"\n");
+        fprintf(dumpfile, "   Device recognized as MTP, no further probing.\n");
+      }
+            usb_close(devh);
+            return 1;
+          }
+  #ifdef LIBUSB_HAS_GET_DRIVER_NP
+    {
+      /*
+       * Specifically avoid probing anything else than USB mass storage devices
+       * and non-associated drivers in Linux.
+       */
+      char devname[0x10];
     
-    for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
-      int k;
-      for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
-        buf[0] = '\0';
-        ret = usb_get_string_simple(devh, 
-				    dev->config[i].interface[j].altsetting[k].iInterface, 
-				    (char *) buf, 
-				    1024);
-	if (ret < 3)
-	  continue;
-        if (strcmp((char *) buf, "MTP") == 0) {
-	  if (dumpfile != NULL) {
-            fprintf(dumpfile, "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
-	    fprintf(dumpfile, "   Interface description contains the string \"MTP\"\n");
-	    fprintf(dumpfile, "   Device recognized as MTP, no further probing.\n");
-	  }
-          usb_close(devh);
-          return 1;
+      devname[0] = '\0';
+      ret = usb_get_driver_np(devh,
+            dev->config[i].interface[j].altsetting[k].iInterface,
+            devname,
+            sizeof(devname));
+      if (devname[0] != '\0' && strcmp(devname, "usb-storage")) {
+        printf("avoid probing device using kernel interface \"%s\"\n", devname);
+        return 0;
+      }
+    }
+  #endif
         }
       }
     }
+  } else {
+    if (dev->descriptor.bNumConfigurations)
+      printf("dev->config is NULL in probe_device_descriptor yet dev->descriptor.bNumConfigurations > 0\n");
   }
   
   /* Read the special descriptor */
@@ -535,11 +566,11 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
  */
 void dump_usbinfo(PTP_USB *ptp_usb)
 {
-  int res;
   struct usb_device *dev;
 
 #ifdef LIBUSB_HAS_GET_DRIVER_NP
   char devname[0x10];
+  int res;
   
   devname[0] = '\0';
   res = usb_get_driver_np(ptp_usb->handle, (int) ptp_usb->interface, devname, sizeof(devname));
@@ -576,7 +607,7 @@ void dump_usbinfo(PTP_USB *ptp_usb)
  * @param ptp_usb the USB device to get suggestion for.
  * @return the suggested playlist extension.
  */
-char const * const get_playlist_extension(PTP_USB *ptp_usb)
+const char *get_playlist_extension(PTP_USB *ptp_usb)
 {
   struct usb_device *dev;
   static char creative_pl_extension[] = ".zpl";
@@ -719,7 +750,9 @@ ptp_read_func (
       result--;
     }
     
-    handler->putfunc(NULL, handler->private, result, bytes, &written);
+    int putfunc_ret = handler->putfunc(NULL, handler->private, result, bytes, &written);
+    if (putfunc_ret != PTP_RC_OK)
+      return putfunc_ret;
     
     ptp_usb->current_transfer_complete += result;
     curread += result;
@@ -786,6 +819,7 @@ ptp_write_func (
     return PTP_ERROR_IO;
   }
   while (curwrite < size) {
+    unsigned long usbwritten = 0;
     towrite = size-curwrite;
     if (towrite > CONTEXT_BLOCK_SIZE) {
       towrite = CONTEXT_BLOCK_SIZE;
@@ -795,19 +829,24 @@ ptp_write_func (
         towrite -= towrite % ptp_usb->outep_maxpacket;
       }
     }
-    handler->getfunc(NULL, handler->private,towrite,bytes,&towrite);
-    result = USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char*)bytes,towrite,ptp_usb->timeout);
+    int getfunc_ret = handler->getfunc(NULL, handler->private,towrite,bytes,&towrite);
+    if (getfunc_ret != PTP_RC_OK)
+      return getfunc_ret;
+    while (usbwritten < towrite) {
+	    result = USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,((char*)bytes+usbwritten),towrite-usbwritten,ptp_usb->timeout);
 #ifdef ENABLE_USB_BULK_DEBUG
-    printf("USB OUT==>\n");
-    data_dump_ascii (stdout,bytes,towrite,16);
+	    printf("USB OUT==>\n");
+	    data_dump_ascii (stdout,bytes+usbwritten,result,16);
 #endif
-    if (result < 0) {
-      return PTP_ERROR_IO;
+	    if (result < 0) {
+	      return PTP_ERROR_IO;
+	    }
+	    // check for result == 0 perhaps too.
+	    // Increase counters
+	    ptp_usb->current_transfer_complete += result;
+	    curwrite += result;
+	    usbwritten += result;
     }
-    // Increase counters
-    ptp_usb->current_transfer_complete += result;
-    curwrite += result;
-
     // call callback
     if (ptp_usb->callback_active) {
       if (ptp_usb->current_transfer_complete >= ptp_usb->current_transfer_total) {
@@ -1025,6 +1064,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 		/* For all camera devices. */
 		datawlen = (size<PTP_USB_BULK_PAYLOAD_LEN_WRITE)?size:PTP_USB_BULK_PAYLOAD_LEN_WRITE;
 		wlen = PTP_USB_BULK_HDR_LEN + datawlen;
+    
 		ret = handler->getfunc(params, handler->private, datawlen, usbdata.payload.data, &gotlen);
 		if (ret != PTP_RC_OK)
 			return ret;
@@ -1134,10 +1174,13 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		}
 		if (usbdata.length == 0xffffffffU) {
 			/* Copy first part of data to 'data' */
+      int putfunc_ret = 
 			handler->putfunc(
 				params, handler->private, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
 				&written
 			);
+      if (putfunc_ret != PTP_RC_OK)
+        return putfunc_ret;
 			/* stuff data directly to passed data handler */
 			while (1) {
 				unsigned long readdata;
@@ -1200,10 +1243,13 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 			params->split_header_data = 1;
 
 		/* Copy first part of data to 'data' */
+    int putfunc_ret = 
 		handler->putfunc(
 			params, handler->private, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
 			&written
 		);
+    if (putfunc_ret != PTP_RC_OK)
+      return putfunc_ret;
     
 		if (FLAG_NO_ZERO_READS(ptp_usb) && 
 		    len+PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
@@ -1632,8 +1678,7 @@ LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
   /* Device has gone since detecting raw devices! */
   if (!found) {
     return LIBMTP_ERROR_NO_DEVICE_ATTACHED;
-  } //*/ // cut to here
-  //////libusb_device = device->libusb_device;
+  }
 
   /* Allocate structs */
   ptp_usb = (PTP_USB *) malloc(sizeof(PTP_USB));
