@@ -62,7 +62,7 @@ struct _PTP_USB {
 
 int open_device (int busn, int devn, short force, PTP_USB *ptp_usb, PTPParams *params, struct usb_device **dev);
 void dump_usbinfo(PTP_USB *ptp_usb);
-char const * const get_playlist_extension(PTP_USB *ptp_usb);
+const char *get_playlist_extension(PTP_USB *ptp_usb);
 void close_device(PTP_USB *ptp_usb, PTPParams *params);
 LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device, 
 					   PTPParams *params,
@@ -70,8 +70,6 @@ LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
 void set_usb_device_timeout(PTP_USB *ptp_usb, int timeout);
 void get_usb_device_timeout(PTP_USB *ptp_usb, int *timeout);
 
-int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile);
-
 /* Flag check macros */
 #define FLAG_BROKEN_MTPGETOBJPROPLIST_ALL(a) \
   ((a)->rawdevice.device_entry.device_flags & DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL)
@@ -105,6 +103,12 @@ int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile);
   ((a)->rawdevice.device_entry.device_flags & (DEVICE_FLAG_PLAYLIST_SPL_V1 | DEVICE_FLAG_PLAYLIST_SPL_V2))
 #define FLAG_CANNOT_HANDLE_DATEMODIFIED(a) \
   ((a)->rawdevice.device_entry.device_flags & DEVICE_FLAG_CANNOT_HANDLE_DATEMODIFIED)
+#define FLAG_BROKEN_SEND_OBJECT_PROPLIST(a) \
+  ((a)->rawdevice.device_entry.device_flags & DEVICE_FLAG_BROKEN_SEND_OBJECT_PROPLIST)
+#define FLAG_BROKEN_BATTERY_LEVEL(a) \
+  ((a)->rawdevice.device_entry.device_flags & DEVICE_FLAG_BROKEN_BATTERY_LEVEL)
+#define FLAG_FLAC_IS_UNKNOWN(a) \
+  ((a)->rawdevice.device_entry.device_flags & DEVICE_FLAG_FLAC_IS_UNKNOWN)
 
 /* connect_first_device return codes */
 #define PTP_CD_RC_CONNECTED	0
