@@ -21,12 +21,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 
- // TODO do I actually use all these?
 #include <stdio.h>
 #include <stdlib.h> // mkstmp()
+#include <unistd.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
 #include <fcntl.h>
 
 #include <string.h>
@@ -90,6 +95,15 @@ int is_spl_playlist(PTPObjectInfo *oi)
          (strcmp((oi->Filename + strlen(oi->Filename) -4), ".spl") == 0);
 }
 
+#ifndef HAVE_MKSTEMP
+# ifdef __WIN32__
+#  include <fcntl.h>
+#  define mkstemp(_pattern) _open(_mktemp(_pattern), _O_CREAT | _O_SHORT_LIVED | _O_EXCL)
+# else
+#  error Missing mkstemp() function.
+# endif
+#endif
+
 /**
  * Take an object ID, a .spl playlist on the MTP device,
  * and convert it to a playlist_t object.
@@ -107,6 +121,8 @@ void spl_to_playlist_t(LIBMTP_mtpdevice_t* device, PTPObjectInfo *oi,
   // Use the Filename as the playlist name, dropping the ".spl" extension
   pl->name = malloc(sizeof(char)*(strlen(oi->Filename) -4 +1));
   memcpy(pl->name, oi->Filename, strlen(oi->Filename) -4);
+  // Set terminating character
+  pl->name[strlen(oi->Filename) - 4] = 0;
   pl->playlist_id = id;
   pl->parent_id = oi->ParentObject;
   pl->storage_id = oi->StorageID;
@@ -250,6 +266,10 @@ int update_spl_playlist(LIBMTP_mtpdevice_t *device,
 
   // read in the playlist of interest
   LIBMTP_playlist_t * old = LIBMTP_Get_Playlist(device, new->playlist_id);
+  
+  // check to see if we found it
+  if (!old)
+    return -1;
 
   // check if the playlists match
   int delta = 0;
